/* Democracy - a Bukkit plugin bringing voting features to Minecraft Servers
 * Copyright (c) 2011 Ed Halley <ed@halley.cc>
 * Offered under the MIT License.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package cc.halley.bukkit.Democracy;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.bukkit.ChatColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

public class DemocracyVoteCommand
    implements CommandExecutor
{
	private Democracy democracy = null;

	public DemocracyVoteCommand(Democracy democracy)
		{ this.democracy = democracy; }

	public boolean onHelp(CommandSender sender)
	{
		sender.sendMessage(ChatColor.GRAY + "usage: /vote [page]: show votes needed");
		sender.sendMessage(ChatColor.GRAY + "usage: /vote <issue> [choice]");
		return true;
	}

	public boolean listMyVoteOpportunities(Player player, int page)
	{
		String who = player.getName();
		List<DemocracyIssue> issues = new ArrayList<DemocracyIssue>();
		Iterator<DemocracyIssue> it = democracy.getVotableIssues(player);
		while (it.hasNext())
		{
			DemocracyIssue issue = it.next();
			// if (!issue.proponent.equals(who)) continue;
			issues.add(issue);
		}

		int PAGE = 5;
		
		page--;
		if (page*PAGE >= issues.size())
		{
			player.sendMessage(ChatColor.GRAY +
			                   "You have no more proposals to vote.");
			return false;
		}
		
		Collections.sort(issues, new Comparator<DemocracyIssue>() {
            public int compare(DemocracyIssue a, DemocracyIssue b)
            	{ return a.name.compareTo(b.name); } } );

		boolean accountable =
			Democracy.config.getBoolean("voting.accountable-vetoes", true);
		
		player.sendMessage(ChatColor.GRAY +
		                   "You can vote on these proposals, " +
		                   ChatColor.GOLD +
		                   "page " + (page+1) +
		                   " of " + ((issues.size()+PAGE-1)/PAGE));
		for (int i = page*PAGE; i < page*PAGE+PAGE; i++)
		{
			if (i >= issues.size())
				break;
			DemocracyIssue issue = issues.get(i);
			player.sendMessage(ChatColor.GRAY +
			                   issue.summarizeStatus(who, accountable));
		}

		return true;
	}

	@Override
	public boolean onCommand(CommandSender sender, Command command,
	                         String label, String[] args)
	{
    	democracy.checkIssuesDue();

    	// Aliases turn "/aye something" into "/vote something aye"
		if (label.equalsIgnoreCase("aye") ||
			label.equalsIgnoreCase("nay"))
		{
			if (args.length > 0)
				return onCommand(sender, command, "vote",
				                 new String[] { args[0], label });
			return onCommand(sender, command, "vote",
			                 new String[] { label });
		}

		// Console users can't vote
		if (!(sender instanceof Player))
		{
			Democracy.log.warning("Only logged-in players can vote.");
			return true;
		}
		Player player = (Player)sender;
		
		// No issue to vote?
		int page = 0;
		try { page = (args.length == 0)? 1 : Integer.parseInt(args[0]); }
		catch (Exception e) { ; }
		if (page > 0)
		{
			listMyVoteOpportunities(player, page);
			return true;
		}

		// Name or subcommand first
		int argp = 0;
		String name = args[argp++];
		
		if (name.equalsIgnoreCase("help"))
			return onHelp(player);

		DemocracyIssue issue = democracy.findIssue(name);
		if (issue == null)
		{
			player.sendMessage(ChatColor.RED +
			                   "Cannot find that proposal to add your vote.");
			return true;
		}

		if (argp >= args.length)
		{
			String who = player.getName();
			boolean accountable =
				Democracy.config.getBoolean("voting.accountable-vetoes", true);
			player.sendMessage(ChatColor.GRAY +
			                   issue.summarizeStatus(who, accountable));
			return true;
		}

		String choice = args[argp++];
		String reason = issue.reasonVoteNotAllowed(player, choice);
		if (reason != null)
		{
			player.sendMessage(ChatColor.RED + reason);
			return true;
		}
		
		issue.castVote(player, choice);
		return true;
	}
}
