
/* 
 * Copyright (c) 2011 Ed Halley <ed@halley.cc>
 * Offered under the MIT License.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package cc.halley.kata;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * A factory of iterators that conditionally select from other iterators.
 * See http://erikras.com/2008/01/18/
 */

public abstract class Filter<T>
{
	public abstract boolean passes(T object);

	public int count(Iterator<T> iterator)
	{
		int n = 0;
		while (iterator.hasNext())
		{
			iterator.next();
			n++;
		}
		return n;
	}

	public Iterator<T> filter(Iterator<T> iterator)
		{ return new FilterIterator(iterator); }

	public Iterable<T> filter(final Iterable<T> iterable)
	{
		return new Iterable<T>() {
			public Iterator<T> iterator()
				{ return filter(iterable.iterator()); } };
	}

	private class FilterIterator
	    implements Iterator<T>
	{
		private Iterator<T> iterator;
		private T next;

		private FilterIterator(Iterator<T> iterator)
		{
			this.iterator = iterator;
			toNext();
		}

		public boolean hasNext()
			{ return next != null; }

		public T next()
		{
			if (next == null) throw new NoSuchElementException();
			T returnValue = next;
			toNext();
			return returnValue;
		}

		public void remove()
			{ throw new UnsupportedOperationException(); }

		private void toNext()
		{
			next = null;
			while (iterator.hasNext())
			{
				T item = iterator.next();
				if (item != null && passes(item))
				{
					next = item;
					break;
				}
			}
		}
	}
}
